-- Init Addon
PoisonTracker = LibStub("AceAddon-3.0"):NewAddon("PoisonTracker", "AceBucket-3.0")

local AceGUI = LibStub("AceGUI-3.0")

--
-- # Constants
--

local VERSION = "0.1"

-- Trackable poisons and their meta data
local ICONT = {
    ["Deadly Poison"] = {
        image = "132290",
        shortname = "dp"
    },
    ["Crippling Poison"] = {
        image = "132274",
        shortname = "cp"
    },
    ["Wound Poison"] = {
        image = "134197",
        shortname = "wp"
    },
    ["Leeching Poison"] = {
        image = "538440",
        shortname = "lp"
    }
}

local ICONSIZE = 65

--
-- # Helpers
--

-- https://stackoverflow.com/questions/2705793/how-to-get-number-of-entries-in-a-lua-table
local function TableLength(T)
    local count = 0
    for _ in pairs(T) do count = count + 1 end
    return count
end

local function TimeToSeconds(T)
    return math.floor(T - GetTime())
end

--
-- # Addon
--

PoisonTracker.frame = CreateFrame("Frame")

--
-- ## When addon first loads
--
function PoisonTracker:OnInitialize()
    self:InitDB()

    local options = {
      name = "PoisonTracker",
      type = 'group',
      args = {
          poisons = {
              name = "Poisons",
              desc = "Enable/disable tracking for single poisons",
              type= "group",
              args = {
                  deadly_poison = {
                    name = "Track Deadly Poison",
                    desc = "Enables / disables tracking for Deadly Poison",
                    type = "toggle",
                    set = function(info,val) self.db.profile.dp = val end,
                    get = function(info) return self.db.profile.dp  end
                  },
                  crippling_poison = {
                    name = "Track Crippling Poison",
                    desc = "Enables / disables tracking for Crippling Poison",
                    type = "toggle",
                    set = function(info,val) self.db.profile.cp = val end,
                    get = function(info) return self.db.profile.cp  end
                  },
                  leeching_poison = {
                    name = "Track Leeching Poison",
                    desc = "Enables / disables tracking for Leeching Poison",
                    type = "toggle",
                    set = function(info,val) self.db.profile.lp = val end,
                    get = function(info) return self.db.profile.lp  end
                  },
                  wound_poison = {
                    name = "Track Wound Poison",
                    desc = "Enables / disables tracking for Wound Poison",
                    type = "toggle",
                    set = function(info,val) self.db.profile.wp = val end,
                    get = function(info) return self.db.profile.wp  end
                  },
              }
          },
          config = {
            name = "Configuration",
            desc = "Global configuration",
            type = "group",
            args = {
                threshold = {
                    name = "Notification Treshold",
                    desc = "Sets the threshold when the notification will be displayed",
                    type = "input",
                    set = function(info,val) self.db.profile.threshold = val end,
                    get = function(info) return self.db.profile.threshold  end
                }
            }
          },
      },
    }

    LibStub("AceConfig-3.0"):RegisterOptionsTable("PoisonTracker", options, {"PoisonTracker", "pt"})

    LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PoisonTracker", "PoisonTracker")
end

--
-- ## Initializes DB
--
function PoisonTracker:InitDB()
    local defaults = {
        profile = {
            dp = true,
            cp = true,
            wp = false,
            lp = false,
            threshold = "10",
        }
    }

    self.db = LibStub("AceDB-3.0"):New("PoisonTracker", defaults, true)
end

--
-- ## When Addon enables or player starts/reloads game
--
function PoisonTracker:OnEnable()
    self.icons = {}

    -- Initialize main container frame
    self.base = AceGUI:Create("SimpleGroup")
    self.base:SetWidth(ICONSIZE)
    self.base:SetHeight(ICONSIZE)
    self.base:SetLayout("Flow")
    self.base:SetPoint("CENTER", UIParent, 0,0)
    self.base.frame:SetFrameStrata("MEDIUM")

    -- Get current class
    class, classFileName, classIndex = UnitClass("player")

    -- Stop execution if player is not rogue
    if classIndex ~= 4 then
        return false
    end

    -- Get current spec
    self.spec = GetSpecialization()

    -- Update spec when user changes talent
    self:RegisterBucketEvent("PLAYER_TALENT_UPDATE", 1, function()
        self.spec = GetSpecialization()
    end)

    -- Fire tick when buffs on player change
    self:RegisterBucketEvent("UNIT_AURA", 1, "Tick")

    -- Fire tick every frame update
    self.frame:SetScript("OnUpdate", function() self:Tick() end)
end

--
-- ## Checks if player is in right settings and fires tracker function for every poison
--
function PoisonTracker:Tick()
    local inInstance = IsInInstance()
    local inPVP = IsInActiveWorldPVP()

    -- Hide UI and cancel execution if user is not in an instance or in the right spec
    if (inInstance == false and inPVP == false) or self.spec ~= 1 then
        for k,v in pairs(self.icons) do
            v.frame:Hide()
        end
        return false
    end

    -- track deadly poision
    if self.db.profile.dp then
        self:Track("Deadly Poison")
    elseif self.icons["Deadly Poison"] then
        self:Reset()
    end

    -- track crippling poision
    if self.db.profile.cp then
        self:Track("Crippling Poison")
    elseif self.icons["Crippling Poison"] then
        self:Reset()
    end

    -- track leeching poision
    if self.db.profile.lp then
        self:Track("Leeching Poison")
    elseif self.icons["Leeching Poison"] then
        self:Reset()
    end

    -- track wound poision
    if self.db.profile.wp then
        self:Track("Wound Poison")
    elseif self.icons["Wound Poison"] then
        self:Reset()
    end
end

--
-- ## Tracks a poison
--
function PoisonTracker:Track(name)
    local aura = {UnitAura("player", name)};

    if aura[1] then
        local time_remaining = TimeToSeconds(aura[7])

        if time_remaining < tonumber(self.db.profile.threshold) then
            self:DrawIcon(name, math.floor(time_remaining) .. " sec", .5)
        elseif self.icons[name] ~= nil and self.icons[name].frame:GetAlpha() > 0 then
            self.icons[name].frame:SetAlpha(0)
        end
    else
        self:DrawIcon(name, nil, 1)
    end
end

--
-- ## Draws or updates an icon into the main container
--
function PoisonTracker:DrawIcon(name, label, opacity)
    if self.icons[name] == nil then
        icon = AceGUI:Create("Icon")
        icon:SetWidth(ICONSIZE)
        icon:SetImageSize(60,60)
        icon:SetImage(ICONT[name]["image"])
        icon:SetPoint("CENTER")
        -- icon:SetCallback("OnEnter", function(widget) widget.frame:SetAlpha(1) end)

        local label = icon.frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight")

        label:SetJustifyH("CENTER")
        label:SetJustifyV("CENTER")
        label:SetPoint("CENTER")
        label:SetHeight(18)
        label:SetText("test")
        icon.text = label

        self.icons[name] = icon
        self.base:AddChild(icon)
        self:ResizeBase()
    else
        icon = self.icons[name]
    end

    if icon.frame:IsVisible() == false then
        icon.frame:Show()
    end

    icon.frame:SetAlpha(opacity)
    icon.text:SetText(label)

    self.icons[name] = icon

    return icon
end

--
-- ## Reset UI
--
function PoisonTracker:Reset()
    for k,v in pairs(self.icons) do
        v.text:Hide()
    end

    self.icons = {}
    self.base:ReleaseChildren()
end

--
-- ## Resize size of main container
--
function PoisonTracker:ResizeBase()
    local c = TableLength(self.icons)
    self.base:SetWidth((c * ICONSIZE) + 1)
    self.base:DoLayout()
end
